﻿Imports System
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Ink
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel


Namespace SimpleSteps
    Public Class MainPageViewModel
        Implements INotifyPropertyChanged

        Private _mgr As NorthwindIBEntities = New NorthwindIBEntities
        Public Property Employees() As ObservableCollection(Of Employee)
            Get
                Return m_Employees
            End Get
            Private Set(ByVal value As ObservableCollection(Of Employee))
                m_Employees = value
            End Set
        End Property
        Private m_Employees As ObservableCollection(Of Employee)
        Public Property Log() As ObservableCollection(Of String)
            Get
                Return m_Log
            End Get
            Private Set(ByVal value As ObservableCollection(Of String))
                m_Log = value
            End Set
        End Property
        Private m_Log As ObservableCollection(Of String)

        Public Sub New()

            Employees = New ObservableCollection(Of Employee)()

            AddHandler Employees.CollectionChanged, AddressOf Employees_CollectionChanged

            Log = New ObservableCollection(Of String)

            _busyPropertyName = "IsBusy"

            AddHandler _mgr.Fetching, Sub(s, e)
                                          WriteToLog(e.Query)
                                          IsBusy = True
                                      End Sub

            AddHandler _mgr.Queried, Sub(s, e)
                                         WriteToLog("Query returned " & e.Results.Cast(Of Object)().Count())
                                         IsBusy = False
                                         _busyPropertyName = "IsOrderBusy"
                                     End Sub
           
            Dim query = _mgr.Employees
          
            query.ExecuteAsync(Sub(op)
                                   op.Results.ForEach(Sub(emp) Employees.Add(emp))
                                   CurrentEmployee = Employees.FirstOrDefault
                               End Sub)

        End Sub

        Private Sub Employees_CollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
            Select Case e.Action
                Case NotifyCollectionChangedAction.Add
                    Add(e.NewItems.Cast(Of Employee)())
                    Exit Select
                Case NotifyCollectionChangedAction.Remove
                    Delete(e.OldItems.Cast(Of Employee)())
                    Exit Select
            End Select
        End Sub

        Private Sub Add(ByVal employees As System.Collections.Generic.IEnumerable(Of Employee))
            If employees.Any(Function(emp) emp.EntityAspect.EntityState.IsDetached()) Then
                _mgr.AddEntities(employees)
            End If
        End Sub

        Private Sub Delete(ByVal employees As IEnumerable(Of Employee))
            employees.ForEach(Sub(emp) emp.EntityAspect.Delete())
            Save()
        End Sub

        Public Sub Save()
            _mgr.SaveChangesAsync()
        End Sub

        Private _CurrentEmployee As Employee
        Public Property CurrentEmployee() As Employee
            Get
                Return _CurrentEmployee
            End Get
            Set(ByVal value As Employee)
                _CurrentEmployee = value
                RaisePropertyChanged("CurrentEmployee")
            End Set
        End Property

        Private Sub WriteToLog(ByVal query As IEntityQuery)
            Dim returnType = query.ElementType
            Dim message As String
            If (returnType.Equals(GetType(Order))) Then
                message = "Fetching Orders from Employee " + CurrentEmployee.FullName
            Else
                message = "Fetching " & Convert.ToString(query.ElementType)
            End If
            WriteToLog(message)
        End Sub

        Private Sub WriteToLog(ByVal message As String)
            Log.Insert(0, message)
        End Sub

        Private _busyPropertyName As String
        Private _isBusy As Boolean
        Public Property IsBusy() As Boolean
            Get
                Return _isBusy
            End Get
            Set(ByVal value As Boolean)
                _isBusy = value
                RaisePropertyChanged(_busyPropertyName)
            End Set
        End Property

        Public ReadOnly Property IsOrderBusy() As Boolean
            Get
                Return _isBusy
            End Get
        End Property

#Region "INotifyPropertyChanged Members"
        Public Event PropertyChanged As PropertyChangedEventHandler Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged

        Private Sub RaisePropertyChanged(ByVal [property] As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs([property]))
        End Sub
#End Region

    End Class
End Namespace