﻿Imports System.Collections.ObjectModel
Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core
Imports System.Collections.Specialized
Imports System.Linq
Imports System.Collections.Generic
Imports System.ComponentModel

Namespace SimpleSteps
    Public Class MainPageViewModel
        Implements INotifyPropertyChanged
        Public Property Employees() As ObservableCollection(Of Employee)
            Get
                Return m_Employees
            End Get
            Private Set(ByVal value As ObservableCollection(Of Employee))
                m_Employees = value
            End Set
        End Property
        Private m_Employees As ObservableCollection(Of Employee)
        Public Property Log() As ObservableCollection(Of String)
            Get
                Return m_Log
            End Get
            Private Set(ByVal value As ObservableCollection(Of String))
                m_Log = value
            End Set
        End Property
        Private m_Log As ObservableCollection(Of String)
        Public Property CurrentEmployee() As Employee
            Get
                Return m_CurrentEmployee
            End Get
            Set(ByVal value As Employee)
                m_CurrentEmployee = value
            End Set
        End Property
        Public m_CurrentEmployee As Employee
        Public Event PropertyChanged As PropertyChangedEventHandler Implements System.ComponentModel.INotifyPropertyChanged.PropertyChanged


        Public Sub New()
            Log = New ObservableCollection(Of String) '()
            WriteToLog("Initializing View Model")
            Employees = New ObservableCollection(Of Employee)()
            AddHandler Employees.CollectionChanged, AddressOf Employees_CollectionChanged
            _mgr = New NorthwindIBEntities()

            AddHandler _mgr.Fetching, Sub(s, e) WriteToLog(e.Query)
            AddHandler _mgr.Fetching, Sub() IsBusy = True
            AddHandler _mgr.Queried, Sub() IsBusy = False
          
            Dim query = _mgr.Employees
            query.ExecuteAsync(Sub(op) op.Results.ForEach(Sub(emp) Employees.Add(emp)))
        End Sub

        Private Sub _mgr_Fetching(ByVal sender As Object, ByVal e As EntityFetchingEventArgs)
            WriteToLog("Fetching " + e.Query.ElementType.ToString)
        End Sub

        Public Sub Save()
            _mgr.SaveChangesAsync()
        End Sub

        Private Sub Employees_CollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
            Select Case e.Action
                Case NotifyCollectionChangedAction.Add
                    Add(e.NewItems.Cast(Of Employee)())
                    Exit Select
                Case NotifyCollectionChangedAction.Remove
                    Delete(e.OldItems.Cast(Of Employee)())
                    Exit Select
            End Select
        End Sub

        Private Sub Add(ByVal employees As IEnumerable(Of Employee))
            employees.ForEach(Sub(emp) _mgr.AddEntity(emp))
        End Sub

        Private Sub Delete(ByVal employees As IEnumerable(Of Employee))
            employees.ForEach(Sub(emp) emp.EntityAspect.Delete())
            Save()
        End Sub

        Private Sub WriteToLog(ByVal message As String)
            Log.Insert(0, message)
        End Sub

        Private Sub WriteToLog(ByVal query As IEntityQuery)
            Dim returnType = query.ElementType
            Dim message = ""
            If (returnType.Equals(GetType(Order))) Then
                message = "Fetching Orders from Employee " + CurrentEmployee.FullName
            Else
                message = "Fetching " & Convert.ToString(query.ElementType)
            End If

            WriteToLog(message)
        End Sub

        Private Sub RaisePropertyChanged(ByVal [property] As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs([property]))
        End Sub

        Public Property IsBusy() As Boolean
            Get
                Return _isBusy
            End Get
            Set(ByVal value As Boolean)
                _isBusy = value
                RaisePropertyChanged("IsBusy")
            End Set
        End Property

        Private _isBusy As Boolean
        Private ReadOnly _mgr As NorthwindIBEntities
    End Class
End Namespace